<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\MAN\Services\FileUploaderService;
use Teknisa\MAN\Services\ParamKDSService;
use Teknisa\MAN\Services\NovoCodigo;
use Teknisa\Model\Util\NovoCodigo as novoCodigoModel;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\MAN\Utilities;
use \Util\Exception;
use Zeedhi\Framework\Util\Functions;

class ParamKDS extends  \Teknisa\MAN\Controllers\TeknisaCrud {
    
    const dataSourceTeclas         = 'vnd09700_kds_teclas',
          dataSourceCores          = 'vnd09700_kds_cores',
          dataSourceSetor          = 'vnd09700_kds_setor',
          dataSourceFilial         = 'vnd09700_kds_filial',
          dataSourceSetorFilial    = 'vnd09700_kds_setor_filial',
          dataSourceSetorFilialRedirect = 'vnd09700_kds_setor_filial_redirect',
          dataSourceArquivos       = 'selectArquivos',
          dataSourceSetorLoja      = 'vnd09700_kds_setor_loja',
          dataSourceProdutoSetor   = 'vnd09700_kds_produtos_setor',
          dataSourceProdObsSetor   = 'vnd09700_kds_prodobs_setor',
          dataSourceIntervalo      = 'vnd09700_kds_intervalos',
          dataSourceAltAutom       = 'vnd09700_kds_produtos_setor_editAut',
          dataSourceLinhaProducao  = 'vnd09700_kds_linhaprod',
          dataSourceLinhaProducaoSetor  = 'vnd09700_kds_linhaprod_setor',
          dataSourceOrdem          = 'vnd09700_kds_ordem_mod',
          dataSourceLinhaPrincipal = 'vnd09700_kds_linha_principal',
          dataSourceFindSetorLoja  = 'vnd09700_kds_setores_loja_disponiveis',
          dataSourceRedirect       = 'vnd09700_kds_redirect',
          dataSourceRedirectSetor  = 'vnd09700_kds_redirect_setor',
          dataSourceProdutosTodosSetores = 'vnd09700_kds_produtos_todos_setores',
          dataSourceSetoresPorProduto = 'vnd09700_kds_setores_por_produto',
          dataSourceSetorTipo = 'vnd09700_kds_setor_tipo',
          dataSourceRotfabrica_produto_departamento = 'rotfabrica_produto_departamento',
          dataSourceGrupromocprod_grupopromoc = 'grupromocprod_grupopromoc',
          dataSourceDesgrupromocpr_prodgrupromoc_produto = 'desgrupromocpr_prodgrupromoc_produto',
          dataSourceRotfablocal_produto_departamento = 'rotfablocal_produto_departamento',
          dataSourceGrupromocprfil_grupopromoc = 'grupromocprfil_grupopromoc',
          dataSourceDesgrupromprfil_prodgrupromoc_produto = 'desgrupromprfil_prodgrupromoc_produto',
          dataSourceSetorLinha = 'vnd09700_kds_linha_setor';

    protected $ParamKDSService,
              $fileUploaderService,
              $novoCodigo,
              $conn,
              $utilService,
              $environment;

    protected $parameterBagColumnsDefault = array('CDSETOR', 'CDFILIAL', 'CDPRODUTO'),
              $parameterBagColumnsSetorLoja = array('CDLOJA', 'CDFILIAL'),
              $parameterBagColmnsIntervalo = array('CDSETOR','CDPRODUTO', 'NRSEQINTERVALO'),
              $parameterBagColumnsLinhaProd = array('CDFILIAL'),
              $parameterBagColumnsLinhaProdSetor = array('CDFILIAL', 'CDLINHAPROD'),
              $parameterBagColumnsLinhaPrincipal = array('CDFILIAL'),
              $parameterBagColumnsOrdem = array('CDFILIAL'),
              $parameterBagColmnsUnidadeKDS = array('CDFILIAL', 'NRATRAPADRAO'),
              $parameterBagColumnsRedirect = array('CDFILIAL', 'IDORIGEMVENDA', 'CDREDIRECIONAMENTO'),
              $parameterBagColumnsRedirectSetor = array('CDSETOR', 'IDTIPOSETOR', 'NRORG'),
              $parameterBagColumnsSetoresPorProduto = array('CDPRODUTO'),
              $parameterBagColumnsSetorTipo = array('IDTIPOSETOR');

    public function __construct ($crudService, $parameterBag, ParamKDSService $ParamKDSService, Manager $dataSourceManager, FileUploaderService $fileUploaderService, NovoCodigo $novoCodigo, $conn, Utilities\Util $utilService, $environment) {
        $this->crudService              = $crudService;
        $this->parameterBag             = $parameterBag;
        $this->ParamKDSService          = $ParamKDSService;
        $this->dataSourceManager        = $dataSourceManager;
        $this->fileUploaderService      = $fileUploaderService;
        $this->novoCodigo               = $novoCodigo;
        $this->conn                     = $conn;
        $this->utilService              = $utilService;
        $this->environment              = $environment;
    }

    public function findSetorPorLoja(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceFindSetorLoja;
        $this->parameterBagColumns = array('CDFILIAL', 'CDLOJA');
        parent::find($request, $response);
    }

    public function findIntervalos(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceIntervalo;
        $this->parameterBagColumns = $this->parameterBagColmnsIntervalo;
        parent::find($request, $response);
    }

    public function deleteIntervalos(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceIntervalo;
        parent::delete($request, $response);
    }

    public function saveIntervalos(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceIntervalo;
        $this->parameterBagColumns = $this->parameterBagColmnsIntervalo;
        parent::save($request, $response);
    }

    public function findTeclas(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceTeclas;
        $this->parameterBagColumns = $this->parameterBagColumnsDefault;
        parent::find($request, $response);
    }

    public function findCores(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceCores;
        $this->parameterBagColumns = $this->parameterBagColumnsDefault;
        parent::find($request, $response);
    }

    public function findSetor(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceSetor;
        
        parent::find($request, $response);
    }

    public function findSetorFilial(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceSetorFilial;
        $this->parameterBagColumns = $this->parameterBagColumnsDefault;
        parent::find($request, $response);
    }
    
    public function findSetorLinha(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceSetorLinha;
        $this->parameterBagColumns = $this->parameterBagColumnsDefault;
        parent::find($request, $response);
    }
    
    public function findSetorFilial_redirect(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceSetorFilialRedirect;
        $this->parameterBagColumns = $this->parameterBagColumnsDefault;
        parent::find($request, $response);
    }

    public function findSetorLoja(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceSetorLoja;
        $this->parameterBagColumns = $this->parameterBagColumnsSetorLoja;
        parent::find($request, $response);
    }

    public function findLinhaProd(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceLinhaProducao;
        $this->parameterBagColumns = $this->parameterBagColumnsLinhaProd;
        parent::find($request, $response);
    }

    public function findLinhaProdSetor(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceLinhaProducaoSetor;
        $this->parameterBagColumns = $this->parameterBagColumnsLinhaProdSetor;
        parent::find($request, $response);
    }

     public function findOrdemMod(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceOrdem;
        $this->parameterBagColumns = $this->parameterBagColumnsOrdem;
        parent::find($request, $response);
    }

    public function findLinhaPrincipal(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceLinhaPrincipal;
        $this->parameterBagColumns = $this->parameterBagColumnsLinhaPrincipal;
        parent::find($request, $response);
    }

     public function findRedirect(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceRedirect;
        $this->parameterBagColumns = $this->parameterBagColumnsRedirect;
        parent::find($request, $response);
    }

     public function findRedirectSetor(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceRedirectSetor;
        $this->parameterBagColumns = $this->parameterBagColumnsRedirectSetor;
        parent::find($request, $response);
    }

    public function findProdutoSetor(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceProdutoSetor;
        $this->parameterBagColumns =$this->parameterBagColumnsDefault;

        $newResponse = new DTO\Response();
        parent::find($request, $newResponse);

        if(empty($newResponse->getDataSets())) {
            $rows = [];
        } else {
            $rows = $newResponse->getDataSets()[0]->getRows();
        }

        for($i = 0; $i < count($rows); $i++) {
            $rows[$i] = $this->returnTimeConcat($rows[$i], 'NRTEMPOPROD');
            $rows[$i] = $this->returnTimeConcat($rows[$i], 'NRTEMPOEXIB');
        }

        $response->addDataSet(new DataSet(self::dataSourceProdutoSetor, $rows));
    }
    
    public function findProdutosTodosSetores(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceProdutosTodosSetores;
        parent::find($request, $response);
    }
    
    public function findSetoresPorProduto(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceSetoresPorProduto;
        $this->parameterBagColumns = $this->parameterBagColumnsSetoresPorProduto;
        parent::find($request, $response);
    }
    
    public function findSetorTipo(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceSetorTipo;
        $this->parameterBagColumns = $this->parameterBagColumnsSetorTipo;
        parent::find($request, $response);
    }

    private function returnTimeConcat ($row, $name) {
        if($row[$name] >= 60) {
            $row[$name.'MIN'] = intVal(trim($row[$name]) / 60);
            $row[$name.'SEC'] = (trim($row[$name]) % 60);
        } else {
            $row[$name.'MIN'] = 0;
            $row[$name.'SEC'] = trim($row[$name]);
        }
        $row[$name.'CONCAT'] = str_pad($row[$name.'MIN'], 2, "0", STR_PAD_LEFT).':'.str_pad($row[$name.'SEC'], 2, "0", STR_PAD_LEFT);
        $row[$name]         =  str_pad($row[$name.'MIN'], 2, "0", STR_PAD_LEFT).':'.str_pad($row[$name.'SEC'], 2, "0", STR_PAD_LEFT);
        return $row;
    }

    public function findProdObsSetor(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceProdObsSetor;
        $this->parameterBagColumns = $this->parameterBagColumnsDefault;
        parent::find($request, $response);
    }
    
    public function findProdutoDepartamento(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $this->dataSourceName = self::dataSourceRotfabrica_produto_departamento;
            $this->parameterBagColumns = array("CDPRODUTO");
            
            parent::find($request, $response);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function findGrupoPromocional(DTO\Request\Filter $request, DTO\Response $response) {
         try {
            $this->dataSourceName = self::dataSourceGrupromocprod_grupopromoc;
            $this->parameterBagColumns = array("CDPRODPROMOCAO");
            
            parent::find($request, $response);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function findDesgrupromocpr_prodgrupromoc_produto(DTO\Request\Filter $request, DTO\Response $response) {
         try {
            $this->dataSourceName = self::dataSourceDesgrupromocpr_prodgrupromoc_produto;
            $this->parameterBagColumns = array("CDPRODPROMOCAO", "CDGRUPROMOC", "NRSEQGRPROMO");
            
            parent::find($request, $response);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function findRotfablocal_produto_departamento(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $this->dataSourceName = self::dataSourceRotfablocal_produto_departamento;
            $this->parameterBagColumns = array("CDPRODUTO", "CDFILIAL");
            
            parent::find($request, $response);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function findGrupromocprfil_grupopromoc(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $this->dataSourceName = self::dataSourceGrupromocprfil_grupopromoc;
            $this->parameterBagColumns = array("CDPRODPROMOCAO", "CDFILIAL");
            
            parent::find($request, $response);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function findDesgrupromprfil_prodgrupromoc_produto(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $this->dataSourceName = self::dataSourceDesgrupromprfil_prodgrupromoc_produto;
            $this->parameterBagColumns = array("CDPRODPROMOCAO", "CDGRUPROMOC", "CDFILIAL", "NRSEQGRPROMOFIL");
            
            parent::find($request, $response);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function save(DTO\Request $request, DTO\Response $response) {
        try {
            $dataSourceName = substr($request->getRoutePath(), 1, strlen($request->getRoutePath()) - 6);
            $rows = $request->getDataSet()->getRows();
            $rows = $rows[count($rows) - 1];

            self::moneyToFloat('QTSUBPRODLOC', $rows);
            self::moneyToFloat('QTPRGRUPROMINFIL', $rows);
            self::moneyToFloat('QTPRGRUPPROMFIL', $rows);
            self::moneyToFloat('QTPRGRUPPROMOC', $rows);
            self::moneyToFloat('QTPRGRUPROMIN', $rows);
            self::moneyToFloat('QTSUBPRODROT', $rows);
            
            $this->dataSourceName = $dataSourceName;
            
            if($dataSourceName === 'rotfabrica_produto_departamento'
                || 'grupromocprod_grupopromoc'
                || 'desgrupromocpr_prodgrupromoc_produto'
                || 'rotfablocal_produto_departamento'
                || 'grupromocprfil_grupopromoc'
                || 'desgrupromprfil_prodgrupromoc_produto') {
                $this->dataSourceName = explode("_", $dataSourceName)[0];
            }

            if($dataSourceName === 'grupromocprfil_grupopromoc') {
                if(is_array($rows))
                    $rows = $rows[0];
                
                if(!Functions::arrayKeyExists('NRSEQGRPROMOFIL', $rows) || empty($rows->get('NRSEQGRPROMOFIL'))) {
                    $rows->set('NRSEQGRPROMOFIL', novoCodigoModel::geraCodigo($rows['NRORG'], "GRUPROMOCPRFIL".$rows['CDFILIAL'].$rows['CDPRODPROMOCAO'].$rows['CDGRUPROMOC'], 1, 10, false, false));
                }
            }
            
            if($dataSourceName === 'grupromocprod_grupopromoc') {
                if(is_array($rows))
                    $rows = $rows[0];

                if(!Functions::arrayKeyExists('NRSEQGRPROMO', $rows) || empty($rows->get('NRSEQGRPROMO'))) {
                    $rows->set('NRSEQGRPROMO', novoCodigoModel::geraCodigo($rows['NRORG'], "GRUPROMOCPROD".$rows['CDPRODPROMOCAO'].$rows['CDGRUPROMOC'], 1, 10, false, false));
                }

                $rows->set('QTPRGRUPPROMOC', floatval(str_replace(',', '.', $rows['QTPRGRUPPROMOC'])));
                $rows->set('QTPRGRUPROMIN', floatval(str_replace(',', '.', $rows['QTPRGRUPROMIN'])));
            }
            
            $newRequest = new DTO\Request\DataSet (
	            new DataSet($dataSourceName, [$rows]),
	            $request->getMethod(),
	            $request->getRoutePath(),
	            $request->getUserId()
            );
            
            parent::save($newRequest, $response);
        }
        catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function saveProdGrupromoc(DTO\Request $request, DTO\Response $response) {
        try {
            $dataSourceName = substr($request->getRoutePath(), 1, strlen($request->getRoutePath()) - 6);
            $rows = $request->getDataSet()->getRows()[0];

            self::moneyToFloat('QTSUBPRODLOC', $rows);
            self::moneyToFloat('QTPRGRUPROMINFIL', $rows);
            self::moneyToFloat('QTPRGRUPPROMFIL', $rows);
            self::moneyToFloat('QTPRGRUPPROMOC', $rows);
            self::moneyToFloat('QTPRGRUPROMIN', $rows);
            self::moneyToFloat('QTSUBPRODROT', $rows);

            $this->dataSourceName = $dataSourceName;

            if($dataSourceName === 'rotfabrica_produto_departamento'
                || 'grupromocprod_grupopromoc'
                || 'desgrupromocpr_prodgrupromoc_produto'
                || 'rotlocal_produto_departamento'
                || 'grupromocprfil_grupopromoc'
                || 'desgrupromprfil_prodgrupromoc_produto') {
                $this->dataSourceName = explode("_", $dataSourceName)[0];
            }

            if(Functions::arrayKeyExists('VRDESPRODPROMOC', $rows)) {
                $validateDataProdGroup = self::validateDataProdGroupP($rows);
                $request = new DTO\Request\DataSet (
    	            new DataSet($dataSourceName, [$rows]),
    	            $request->getMethod(),
    	            $request->getRoutePath(),
    	            $request->getUserId()
                );
                 
                if($validateDataProdGroup) {
                    parent::delete($request, $response);
                } else {
                    parent::save($request, $response);
                }
            }
            
            if(Functions::arrayKeyExists('VRDESPRPROMOFIL', $rows)) {
                $validateDataProdGroup = self::validateDataProdGroupL($rows);
                if($validateDataProdGroup){
                    parent::delete($request, $response);
                } else {
                    parent::save($request, $response);
                }
            }
        }
        catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function saveBlob(DTO\Request $request, DTO\Response $response) {
    	try {
            $row = $request->getRow();
            
            if (isset($row["TXROTFAB"])) {
                    $row['__is_new'] = !$this->ParamKDSService->checkIsNewPadrao($row);
                    $row["TXROTFAB"] = bin2hex($row["TXROTFAB"]);
                    $this->dataSourceName = "descroteiro";
            } else {
                if(isset($row["TXROTFABLOC"])) {
                    $row['__is_new'] = !$this->ParamKDSService->checkIsNewLocal($row);
                    $row["TXROTFABLOC"] = bin2hex($row["TXROTFABLOC"]);
                    $this->dataSourceName = "descrotlocal";
                } else {
                    throw Exception("Row passada não possui campo BLOB");
                }
            }
            
            $newRequest = new DTO\Request\Row($row, '', '', '');
    	    parent::save($newRequest, $response);
    	} catch(\Exception $e) {
    	    $response->setError(new DTO\Response\Error($e->getMessage(),1));
    	}
    }
    
    public function checkForRecursiveComposition(DTO\Request\Row $request, DTO\Response $response) {
        // 'CDPRODUTOPAI' -> Código do produto pai.
        // 'CDPRODUTOFILHO' -> Código do produto filho.
        $row = $request->getRow();
        $this->compositionProducts = array();

        $produtosFilhos = array_merge_recursive(array($row['CDPRODUTOFILHO']), array());

        foreach($produtosFilhos as $cdprodutofilho) {
            $row['CDPRODUTOFILHO'] = $cdprodutofilho;
            $this->buildCompositionArray($request, $row);
        }

        $check = $this->isARecursiveComposition($row['CDPRODUTOPAI']);

        unset($this->compositionProducts);

        $response->addDataSet(new DataSet('isRecursiveComposition', [$check]));
    }
    
    private function isARecursiveComposition($cdproduto) {
        return in_array($cdproduto, $this->compositionProducts);
    }
    
    private function buildCompositionArray ($request, $row) {
        $products = $this->getCompositionItems($request, $row);

        if(count($products) === 0 ) {
            array_push($this->compositionProducts, $row['CDPRODUTOFILHO']);
        } else {
            foreach ($products as $product) {
                array_push($this->compositionProducts, $product['CDPRODUTO']);
                if(!empty($product['CDSUBPRODLOC']) && !$this->isARecursiveComposition($product['CDSUBPRODLOC']) ) {
                    $row['CDPRODUTOFILHO'] = $product['CDSUBPRODLOC'];
                    $this->buildCompositionArray($request, $row);
                }
                if(!empty($product['CDSUBPRODUTO']) && !$this->isARecursiveComposition($product['CDSUBPRODUTO'])) {
                    $row['CDPRODUTOFILHO'] = $product['CDSUBPRODUTO'];
                    $this->buildCompositionArray($request, $row);
                }
            }
        }
    }
    
    private function getCompositionItems($request, $row) {
        $products = array();

        if(!Functions::arrayKeyExists('CDFILIAL', $row)) {
            $composicaoPadraoFilter['CDPRODUTO'] = $row['CDPRODUTOFILHO'];

            $this->parameterBagColumns = array("CDPRODUTO");
            $this->dataSourceName = 'rotfabrica_produto_departamento';

            $products = array_merge($products, $this->genericFindComposition($request, $composicaoPadraoFilter));
        } else {
            $composicaoLocalFilter['CDPRODUTO'] = $row['CDPRODUTOFILHO'];
            $composicaoLocalFilter['CDFILIAL']  = $row['CDFILIAL'];

            $this->dataSourceName = 'rotfablocal_produto_departamento';
            $this->parameterBagColumns = array("CDPRODUTO", "CDFILIAL");

            $products = array_merge($products, $this->genericFindComposition($request, $composicaoLocalFilter));
        }

        return $products;
    }
    
    private function genericFindComposition($request, $row) {
        $products = array();

        $newRequest = $this->createNewZeedhiFilterRequestObject($request, $this->dataSourceName, $row);

        $response = new Response();

        parent::find($newRequest, $response);

        $dataSets = $response->getDataSets();

        if(count($dataSets)) {
            $products = $dataSets[0]->getRows();
        }

        return $products;
    }
    
    private function createNewZeedhiFilterRequestObject($request, $dataSourceName, $conditions) {
        foreach($conditions as $key => $value) {
            $dataSourceFilter[] = $condition = array(
                "columnName" => $key,
                "operator"   => "=",
                "value"      => $value
            );
        }

        return new DTO\Request\Filter(
            new FilterCriteria(
                $dataSourceName,
                $dataSourceFilter,
                1,
                500
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
    }
    
    public function deleteGrupoPromocional(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->ParamKDSService->deleteGrupoPromocional($row);  
        } catch(\Exception $e) {
           $response->setError(new Error('Não é possível excluir pois há registros filhos com descontos.', 1));
        }
    }
    
    private function moneyToFloat($name, &$rows) {
        if(Functions::arrayKeyExists($name, $rows)) {
            if(gettype($rows[$name]) == 'string') {
                if(!!$rows[$name]) {
        			$rows[$name] = preg_replace("/[^0-9,]/", "", $rows[$name]);
        			$rows[$name] = str_replace ("," , "." , $rows[$name]);
        		} else {
        			$rows[$name] = 0;
        		}
            }
        }
    }
    
    private function validateDataProdGroupP(&$rows) {
        $retorno = false;
        
        if(!$rows['VRDESPRODPROMOC']) {
            if($rows['IDAPLICADESCPR'] == 'T' && $rows['IDPRODPRESELEC'] == 'N' && $rows['IDPRINCPROD'] == 'N' && $rows['NRORDPROMOPR'] == null) {
                $retorno = true;
            } else {
                $rows['VRDESPRODPROMOC'] = '0';
                $rows['IDDESCACRPROMO'] = 'D';
                $rows['IDPERVALORDES'] = 'V';
            }
        }
        return $retorno;
    }

    private function validateDataProdGroupL(&$rows) {
        $retorno = false;
        
        if(!$rows['VRDESPRPROMOFIL']) {
            if($rows['IDAPLICADESCPRFI'] == 'T' && $rows['IDPRODPRESELFIL'] == 'N' && $rows['IDPRINCPROD'] == 'N' && $rows['NRORDPROMOPRFIL'] == null) {
                $retorno = true;
            } else {
                 $rows['VRDESPRPROMOFIL'] = '0';
                $rows['IDDESCACRPROFIL'] = 'D';
                $rows['IDPERVALDESFIL'] = 'V';
            }
        }
        return $retorno;
    }

    public function saveSetor(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceSetor;
        $row = $request->getDataSet()->getRows()[0];
        if ($row["IDTIPOSETOR"] === "E") {
            $this->ParamKDSService->atualizarHabilitacao($row);
        }
        $this->parameterBagColumns = $this->parameterBagColumnsDefault;
        parent::save($request, $response);
    }

    public function saveTeclas(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceTeclas;
        $this->parameterBagColumns = $this->parameterBagColumnsDefault;
        parent::save($request, $response);
    }

    public function saveCores(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceCores;
        $this->parameterBagColumns = $this->parameterBagColumnsDefault;
        parent::save($request, $response);
    }

    public function saveLinhaProd(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceLinhaProducao;
        $this->parameterBagColumns = $this->parameterBagColumnsLinhaProd;
        $conditions = $this->utilService->getParamsAsArray($request);
        $Hundred = $this->ParamKDSService->somaLinhaPrincipal($conditions[0]["CDLINHAPROD"], $conditions[0]["CDFILIAL"]);
        $Hundred += $conditions[0]["NRPERCDEMANDA"];
        
        if($Hundred > 100){
            throw new \Exception("As somas das demandas não podem ser superior a 100%! ");
        }
        
        if($conditions[0]["IDLINHAPRINCIPAL"] == 'S') {
            $this->ParamKDSService->deleteLinhasetorprincipal($conditions[0]["CDFILIAL"]);
            $this->ParamKDSService->setLinhasetorprincipal($conditions[0]["CDLINHAPROD"], $conditions[0]["CDFILIAL"]);
            $this->ParamKDSService->setLinhaPrincipal($conditions[0]["CDLINHAPROD"], $conditions[0]["CDFILIAL"]);
        }
        
        parent::save($request, $response);
    }

    public function saveLinhaProdSetor(DTO\Request $request, DTO\Response $response) {
        try {
            $this->dataSourceName = self::dataSourceLinhaProducaoSetor;
            $this->parameterBagColumns = $this->parameterBagColumnsLinhaProdSetor;
            $conditions = $this->utilService->getParamsAsArray($request);
            $faltasetor = $this->ParamKDSService->getSetorprincipal($conditions[0]["CDFILIAL"]);

            // coloca o setor correspondente como null caso não tenha sido selecionado algum
            if(!isset($conditions[0]["CDSETORCORRESP"])) {
                $conditions[0]["CDSETORCORRESP"] = null;
            }

            if (empty($faltasetor) && $conditions[0]["IDLINHAPRINCIPAL"] != 'S') {
                throw new \Exception("Não há setores cadastrados na linha principal. Cadastre um setor na linha principal!");
            }
            
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function saveOrdemMod(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceOrdem;
        $this->parameterBagColumns = $this->parameterBagColumnsOrdem;
        parent::save($request, $response);
    }

    public function saveRedirect(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceRedirect;
        $this->parameterBagColumns = $this->parameterBagColumnsRedirect;
        $conditions = $this->utilService->getParamsAsArray($request);
        $same = $this->ParamKDSService->getSameRedirect($conditions[0]["CDFILIAL"],$conditions[0]["CDSETOR"]);
        $ans = $this->ParamKDSService->getExistsRedirect($conditions[0]["CDFILIAL"],$conditions[0]["CDSETOR"],$conditions[0]["CDSETORRED"]);
        $valido = $this->ParamKDSService->validRedirec($conditions[0]["CDFILIAL"],$conditions[0]["CDSETOR"], $conditions[0]["CDSETORRED"], $conditions[0]["IDORIGEMVENDA"],$conditions[0]["IDHABILITADO"]);
        
        try{
            if(($conditions[0]["__is_new"]) == true && !empty($valido)){
                throw new \Exception("Redirecionamento já cadastrado!");
                exit;
            } else if(($conditions[0]["changed"]) && !empty($valido)){
                if (($ans[0]["CDREDIRECIONAMENTO"] === $conditions[0]["CDREDIRECIONAMENTO"])){
                    exit;
                } else {
                    throw new \Exception("Há redirecionamentos cadastrados com os parâmetros informados!");
                        exit;
                }
            } else if((count($same) != 1)){
                if(isset($ans[0])){
                    if(($ans[0]["CDREDIRECIONAMENTO"] === $conditions[0]["CDREDIRECIONAMENTO"]) && ($conditions[0]["__is_new"]) == true) {
                        throw new \Exception("Há redirecionamentos cadastrados com os parâmetros informados!");
                        exit;
                    }
                }
                
                if(($conditions[0]["changed"])&&($conditions[0]["__is_new"]) != true){
                    $this->ParamKDSService->setRedirect($conditions[0]["CDSETOR"],
                    $conditions[0]["CDSETORRED"], $conditions[0]["IDORIGEMVENDA"],$conditions[0]["CDFILIAL"],
                    $conditions[0]["CDREDIRECIONAMENTO"],$conditions[0]["IDHABILITADO"]);
                } else {
                   parent::save($request, $response); 
                }
                
            } else { 
                throw new \Exception("Setor de origem já cadastrado como setor de destino!");
                exit;
            }
            
        }catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
        
    }

    public function saveFilial(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceFilial;
        $row = $request->getDataSet()->getRows()[0];

        if ($row["changed"]) {
            $this->ParamKDSService->updateUnidadeKDS($row);
        }
        return;
    }

    public function saveSetorFilial(DTO\Request $request, DTO\Response $response) {
        $newResponse = new DTO\Response;
        $rows = $request->getDataSet()->getRows();
        if($rows[0]["IDATIVO"]=="N"){
            $row = $rows[0];
        $result   = $this->ParamKDSService->SetorUtilizadoRedirecionamneto($row);
            if($result){
                foreach($result as $excluir){
                    $this->ParamKDSService->deleteSetorRedirecionamento($excluir);
                }
            }
        
        }
        
        if(count($rows) === 1) {
            $row = $rows[0];
            $bool = $row->get('__is_new');

            $row->set('__is_new', true);

            if(!!$row['SOMBEEP'] && !is_string($row['SOMBEEP']) && count($row['SOMBEEP']) > 0 ) {
                foreach ($row['SOMBEEP'] as $file) {

                    $row->set('NMARQUIVO', $file['name']);
                    $row->set('TPARQUIVO', $file['type']);

                    // KDS uses only the base64.
                    // $metaData = $base64withMIMEDivided[0];
                    $base64withMIMEDivided = explode(",", $file['b64File']);
                    $base64 = $base64withMIMEDivided[1];
                    $row->set('ARQUIVO', $base64);
                    $NRORG = $this->environment->getNrOrgTrab();
                    $size = 1;
                    $row->set('CDARQUIVO', str_pad($this->novoCodigo->geraCodigo($this->conn, 'ARQUIVO', $NRORG, $size), 20, "0", STR_PAD_LEFT) );

                    $this->dataSourceName = self::dataSourceArquivos;
                    parent::save($request, $newResponse);
                }

                $this->fileUploaderService->setImageForSave($row, 'CDARQUIVO', 'SOMBEEP');
            }

            $row->set('__is_new', $bool);

            if (empty($row["CDARQUIVO"])) {
                $row["CDARQUIVO"] = "";
            }

            $this->dataSourceName = self::dataSourceSetorFilial;
            $this->parameterBagColumns = $this->parameterBagColumnsSetorLoja;
            parent::save($request, $response);
        }
    }

    public function saveSetorLoja(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceSetorLoja;
        $this->parameterBagColumns = $this->parameterBagColumnsDefault;
        $row = $request->getDataSet()->getRows()[0];

        $params = array(
			':CDFILIAL'     => $row['CDFILIAL'],
			':CDLOJA'    	=> $row['CDLOJA'],
			':CDSETOR'	    => $row['CDSETOR'],
			':IDTIPOSETOR'  => $row['IDTIPOSETOR']
		);

        // verifica se o tipo do setor é de Montagem e salva
        if($row['IDTIPOSETOR'] == 'M') {
            parent::save($request, $response);
        } else {
    		$podeSalvar = $this->ParamKDSService->verificarProdutosSetor($params);

    		if(gettype($podeSalvar) == 'boolean' && $podeSalvar) {
                parent::save($request, $response);
    		} else {
                $response->addDataSet(new DataSet("produtos_conflito", $podeSalvar));
    		}
        }
    }

    public function saveProdutoSetor(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceProdutoSetor;
        $row = $request->getDataSet()->getRows()[0];

        $CDSETOR = $row['CDSETOR'];

        if(isset($row['NRTEMPOPROD']) || isset($row['NRTEMPOEXIB'])) {
            $row->set('NRTEMPOPROD', (((intval(substr($row['NRTEMPOPROD'], 0, 2))) * 60) + intval(substr($row['NRTEMPOPROD'], 3, 2))) . '');
            $row->set('NRTEMPOEXIB', (((intval(substr($row['NRTEMPOEXIB'], 0, 2))) * 60) + intval(substr($row['NRTEMPOEXIB'], 3, 2))) . '');
        }

        if($row['IDRESUMOPRODKDS'] == "DONTSHOW" || empty($row['IDRESUMOPRODKDS']))
            $row['IDRESUMOPRODKDS'] = null;

        if($row['IDRESTRICAOORIGEM'] == "NULL" || empty($row['IDRESTRICAOORIGEM']))
            $row['IDRESTRICAOORIGEM'] = null;

        $CDPRODUTO = $row['CDPRODUTO'];
        $CDARVPROD = $row['CDARVPROD'];
        $isNew  = $row['__is_new'];

        if (isset($row['CDPRODUTOFINAL'])) {
            $CDPRODUTOFINAL = $row['CDPRODUTOFINAL'];
            $CDARVPRODFINAL = $row['CDARVPRODFINAL'];
        } else {
            $CDPRODUTOFINAL = $row['CDPRODUTO'];
            $CDARVPRODFINAL = $row['CDARVPROD'];
        }

        if($isNew || ($CDARVPROD != $CDARVPRODFINAL))
            $prodArray = $this->ParamKDSService->GetProdInterval($CDARVPROD, $CDARVPRODFINAL, $CDSETOR, $isNew);
        else
            $prodArray = [array('CDPRODUTO' => $row['CDPRODUTO'])];

        foreach ($prodArray as $product) {
            $row['CDPRODUTO'] = $product['CDPRODUTO'];
            parent::save($request, $response);
        }
    }

    public function saveProdObsSetor(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceProdObsSetor;
        $row = $request->getDataSet()->getRows()[0];
        if($request->getDataSet()->getRows()[0]['__is_new'] == true){
            foreach ($request->getDataSet()->getRows()[0]['CDOCORR'] as $value) {
                $row['CDOCORR'] = $value;
                parent::save($request, $response);
            }
        }else{
            parent::save($request, $response);
        }
    }
    
    public function saveConfigProdutoSetor(DTO\Request $request, DTO\Response $response) {
        $conditions = $this->utilService->getParamsAsArray($request);
        
        $params = array(
            ":IDTIPOSETOR"       => $conditions[0]['IDTIPOSETOR'],
            ":DSAPELIDORESUMO"   => $conditions[0]['DSAPELIDORESUMO'],
            ":NRTEMPOEXIB"       => $conditions[0]['NRTEMPOEXIB'],
            ":NRTEMPOPROD"       => $conditions[0]['NRTEMPOPROD'],
            ":IDRESUMOPRODKDS"   => ($conditions[0]['IDRESUMOPRODKDS']) === "DONTSHOW" ? null : $conditions[0]['IDRESUMOPRODKDS'],
            ":IDRESTRICAOORIGEM" => ($conditions[0]['IDRESTRICAOORIGEM']) === "API" ? $conditions[0]['IDRESTRICAOORIGEM'] : null,
            ":IDIMPRPRODPICK"    => $conditions[0]['IDIMPRPRODPICK'],
            ":CDSETOR"           => ($conditions[0]['SETORES']) ? null : $conditions[0]['CDSETOR'],
            ":CDPRODUTO"         => $conditions[0]['CDPRODUTO']
        );
        
        $this->ParamKDSService->saveConfigProdutoSetor($params);
    }
    
    public function exportaConfigProdutoParaSetores(DTO\Request\Row $request, DTO\Response $response) {
        $dataExport = $request->getRow();
        
        $setoresDestino = $dataExport['CDSETORDESTINO'];
        
        $paramsProduto = array(
            ":CDPRODUTO" => $dataExport['CDPRODUTO'],
            ":CDSETOR"   => $dataExport['CDSETOR']
        );
        
        // select produto
        $produto = $this->ParamKDSService->selectProdutoSetor($paramsProduto);
        
        // deleta a configuração do produto nos outros setores e faz o insert
        $this->ParamKDSService->insertConfigProdutos($produto, $setoresDestino);
    }

    public function deleteTeclas(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceTeclas;
        parent::delete($request, $response);
    }

    public function deleteCores(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceCores;
        parent::delete($request, $response);
    }

    public function deleteSetor(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceSetor;
        parent::delete($request, $response);
    }
    
    public function verificarSetorFilial(DTO\Request $request, DTO\Response $response) {
        $row = $this->utilService->getParamsAsArray($request);
        if($this->ParamKDSService->totalSetoresPorLoja($row) !== 0){
            $response->addMessage(new Message("Esse setor está vinculado a uma ou mais lojas.", "E"));
        } elseif ($this->ParamKDSService->totalLinhaSetor($row) !== 0) {
            $response->addMessage(new Message("Esse setor está vinculado a uma ou mais linhas de produção.", "E"));
        } else {
            $this->deleteSetorFilial($request,$response);
        }
    }

    private function deleteSetorFilial (DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceSetorFilial;
        parent::delete($request, $response);
    }

    public function deleteLinhaProd (DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceLinhaProducao;
        $conditions = $this->utilService->getParamsAsArray($request);
        $this->ParamKDSService->deleteLinhasetor($conditions["CDLINHAPROD"], $conditions["CDFILIAL"]);
        parent::delete($request, $response);
    }

    public function deleteLinhaProdSetor (DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceLinhaProducaoSetor;
        $conditions = $this->utilService->getParamsAsArray($request);
        $this->ParamKDSService->deleteSetorprincipal($conditions["CDSETOR"], $conditions["CDFILIAL"]);
        parent::delete($request, $response);
    }

    public function deleteOrdemMod (DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceOrdem;
        parent::delete($request, $response);
    }

    public function deleteRedirect (DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceRedirect;
        parent::delete($request, $response);
    }

    public function deleteSetorLoja(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceSetorLoja;
        parent::delete($request, $response);
    }

    public function deleteProdutoSetor(DTO\Request\Row $request, DTO\Response $response) {
        $row = $this->utilService->getParamsAsArray($request);
        $this->ParamKDSService->deletarTodasObservacoesIntervalosProduto($this->conn, $row);
        $this->dataSourceName = self::dataSourceProdutoSetor;
        parent::delete($request, $response);
    }

    public function deleteMultiplo ($request, $response) {
        try {
            $dataSet = $request->getDataSet();

            $rows = $dataSet->getRows();

            foreach ($rows as $row) {
                $this->ParamKDSService->deletarTodasObservacoesIntervalosProduto($this->conn, $row);
            }

            $this->dataSourceName = self::dataSourceProdutoSetor;
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
            $nrDeletedRows = count($deletedRows);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function deleteProdObsSetor(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceProdObsSetor;
        parent::delete($request, $response);
    }

    public function editAutomatico (DTO\Request $request, DTO\Response $response) {
         $this->dataSourceName = self::dataSourceAltAutom;
         $row = $request->getDataSet()->getRows()[0];
         $nrProdutos = count($row['CDPRODUTO_BK']);

         for ($i = 0; $i < $nrProdutos; $i++) {
             $row['__is_new'] = false;
             $row['CDSETOR'] = $row['CDSETOR'];
             $row['CDPRODUTO'] = $row['CDPRODUTO_BK'][$i];
             $row['NRTEMPOPROD'] = $row['NRTEMPOPROD_BK'][$i];
             $row['NRTEMPOEXIB'] = $row['NRTEMPOEXIB_BK'][$i];
             $row['IDRESUMOPRODKDS'] = $row['IDRESUMOPRODKDS_BK'][$i];

             if($row['IDRESTRICAOORIGEM_BK'][$i] == 'NULL') {
                $row['IDRESTRICAOORIGEM'] = null;
             } else
                $row['IDRESTRICAOORIGEM'] = $row['IDRESTRICAOORIGEM_BK'][$i];

            if($row['IDRESUMOPRODKDS_BK'][$i] == 'DONTSHOW') {
                $row['IDRESUMOPRODKDS'] = null;
             } else
                $row['IDRESUMOPRODKDS'] = $row['IDRESUMOPRODKDS_BK'][$i];

             if($row['NRTEMPOPROD'] == 'NaN' || $row['NRTEMPOEXIB'] == 'NaN') {
                 $response->addMessage(new Message("Ocorreu um erro e a execucao da alteracao foi cancelada.", "E"));
                 break;
             } else {
                parent::save($request, $response);
             }
         }

        if($i != $nrProdutos)
            $response->addMessage(new Message("Nao foi possivel alterar todos os produtos. Produtos alterados:  $i", "E"));
    }

    public function deleteProdObsSetorMultiplo(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceProdObsSetor;
        parent::delete($request, $response);
    }

    public function findAlteracaoAut (DTO\Request $request, DTO\Response $response) {

    }

    public function editarProdutoTodosSetores (DTO\Request $request, DTO\Response $response) {
        $row = $this->utilService->getParamsAsArray($request);

        $backUpTempProd = $row['NRTEMPOPROD'];
        $backUpTempExib = $row['NRTEMPOEXIB'];

        $params = array(
            ':CDPRODUTO'    => $row['CDPRODUTO']
		);

		$setores = $this->ParamKDSService->getSetoresProducao($params);

        $method = "saveProdutoSetor";
        $routePath = $request->getRoutePath();
        $userId = $request->getUserId();

		foreach ($setores as $setor) {
            $row['CDSETOR'] = $setor['CDSETOR'];
            $row['NRTEMPOPROD'] = $backUpTempProd;
            $row['NRTEMPOEXIB'] = $backUpTempExib;
            $dataSet = new DataSet(self::dataSourceProdutoSetor, array($row));
		    $request = new DTO\Request\DataSet($dataSet, $method, $routePath, $userId);
		    $this->saveProdutoSetor($request, $response);
        }
    }

    public function importarProdutos (DTO\Request $request, DTO\Response $response) {
        $row = $this->utilService->getParamsAsArray($request);

	    $params = array(
            ':CDSETOR' => $row['CDSETOR']
		);

		$cdsetor = $row['CDSETOR_DESTINO'];

        $arrayProdutos = array();
        $produtosResultQuery = $this->ParamKDSService->getProdutosSetor($params);

        $params[':CDSETOR'] = $row['CDSETOR_DESTINO'];
        $produtosJaExistentes = $this->ParamKDSService->getProdutosSetor($params);

        for ($i = 0; $i < sizeof($produtosJaExistentes); $i++) {
            array_push($arrayProdutos, $produtosJaExistentes[$i]['CDPRODUTO']);
        }

        $produtos = $produtosResultQuery;
        $qtdeProdutos = sizeof($produtos);

        for ($i = 0; $i < $qtdeProdutos; $i++) {
            $cdproduto = $produtos[$i]['CDPRODUTO'];

            if(!in_array($cdproduto, $arrayProdutos)) {
                $nrtempoprod = $produtos[$i]['NRTEMPOPROD'];
                $nrtempoexib = $produtos[$i]['NRTEMPOEXIB'];
                $idresumoprodkds = $produtos[$i]['IDRESUMOPRODKDS'];
                $idrestricaoorigem = $produtos[$i]['IDRESTRICAOORIGEM'];
                $dsapelidoresumo = $produtos[$i]['DSAPELIDORESUMO'];
                $idimprprodpick = $produtos[$i]['IDIMPRPRODPICK'];

                $params = array(
                    ':CDSETOR'            => $cdsetor,
        			':CDPRODUTO'          => $cdproduto,
        			':NRTEMPOPROD'        => $nrtempoprod,
        			':NRTEMPOEXIB'        => $nrtempoexib,
        			':IDRESUMOPRODKDS'    => $idresumoprodkds,
        			':IDRESTRICAOORIGEM'  => $idrestricaoorigem,
        			':DSAPELIDORESUMO'    => $dsapelidoresumo,
        			':IDIMPRPRODPICK'     => $idimprprodpick
    		    );

    		    $this->ParamKDSService->insertProdutoSetor($params);

    		    $params = array(
    		        ':CDSETOR'            => $row['CDSETOR'],
        	    	':CDPRODUTO'          => $cdproduto,
        	    	':CDSETOR_DESTINO'    => $cdsetor
    		    );

    		    if($row['IMPORTAR_OBSERVACAO'] == 'S'){
    		        $this->insertObsProduto($params);
    		    }

    		    if($row['IMPORTAR_INTERVALO']=='S'){
    		        $this->insertIntervaloProduto($params, $row['NRORG']);
    		    }

            }

        }
        return $response;
    }

    private function insertIntervaloProduto($params, $nrorg){
        $intervalos = $this->ParamKDSService->getIntervalosProduto($params);

        foreach ($intervalos as $intervalo) {
            $intervalo['NRSEQINTERVALO'] = $this->novoCodigo->getCodeByTableName('SETORPRODINTER', $nrorg);
            $intervalo['CDSETOR'] = $params[':CDSETOR_DESTINO'];
            $this->ParamKDSService->insertIntervaloProduto($intervalo, $nrorg);
        }
    }

    private function insertObsProduto($params){
        $obs = $this->ParamKDSService->getObservacaoProduto($params);

        foreach ($obs as $item) {
	        $item['CDSETOR'] = $params[':CDSETOR_DESTINO'];
            $this->ParamKDSService->insertObsProduto($item);
        }
    }

    public function importarSetor (DTO\Request\Row $request, DTO\Response $response){

        try{
            $rows = $this->utilService->getParamsAsArray($request);
            $numerosLinhas = $this->ParamKDSService->importarSetor($rows);
            $response->addDataSet(new DataSet('ImportarProdutoRepo', array('total'=>$numerosLinhas)));
        }catch(\Exception $e){
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function delete(DTO\Request $request, DTO\Response $response) {
        try {
            $rota = substr($request->getRoutePath(), 1, strlen($request->getRoutePath()) - 8);
            $data = explode("_", $rota);
            $dataSource = $data[0];
            $filhos = false;
            $row = array();
            $primaryKeys = array();

            if($dataSource === 'grupromocprfil' || $dataSource === 'grupromoc' || $dataSource === 'grupromocprod'){
                $filhos = true;
            }

            //---------------------------Informa quais são as tabelas com registros filhos-----------------------//
            if($dataSource === 'grupromoc') {
                $dataSourceFilhos = array(array('name' => 'prodgrupromoc', 'filhos' => 'false'),
                                          array('name' => 'grupromocprod', 'filhos' => 'false'),
                                          array('name' => 'grupromocprfil', 'filhos' => 'false'));
            }

            if($dataSource === 'prodgrupromoc') $dataSourceFilhos = array(array('name' => 'desgrupromocpr', 'filhos' => 'false'),
                                                                    array('name' => 'desgrupromprfil', 'filhos' => 'false'));

            if($dataSource === 'grupromocprod') $dataSourceFilhos = array(array('name' => 'desgrupromocpr', 'filhos' => 'false'));

            if($dataSource === 'grupromocprfil') $dataSourceFilhos = array(array('name' => 'desgrupromprfil', 'filhos' => 'false'));


            //-----------------------------------------------------------------------------------------------------//

            $row = $primaryKeys;
            
            if($filhos === true) {
               foreach($primaryKeys as $key) {
                    $requestGetRow = $request->getRow();
                    $row[$key] = $requestGetRow[$key];
               }

                $this->deleteFilhos($dataSourceFilhos, $primaryKeys, $row, $response);
            }

            $this->dataSourceName = $dataSource;
            parent::delete($request, $response);

        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function deleteFilhos($dataSourceFilhos, $primaryKeys, $row, $response) {
        // Deleta Filhos

        // Percorre o vetor de dataSourceFilhos
        try {
            for($i = 0; $i < count($dataSourceFilhos); $i++) {
                // Verifica se os filhos possuem filhos e chama o metado novamente caso tenham

                if($dataSourceFilhos[$i]['filhos'] === 'true') {
                    $this->deleteFilhos($dataSourceFilhos[$i]['dataSourceFilhos'], $primaryKeys, $row, $response);
                }

                $conditions = $this->crudService->getConditionsByColumns($primaryKeys, $row);

                $dataSet = $this->crudService->getFilteredDataSet($dataSourceFilhos[$i]['name'], $conditions);
                $this->crudService->delete($response, $dataSet);
            }
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function AltAuto(DTO\Request $request, DTO\Response $response){
        try {
            $rows = $request->getDataSet()->getRows();
            $row = $rows[count($rows) - 1];
            $this->dataSourceName = "grupromocprod";
            
            foreach($row["CDGRUPROMOC"] as $key => $CDGRUPO) {
                $NRSEQGRPROMO = $row['NRSEQGRPROMO'][$key];
                
                if(!isset($row["NRORDPROMOGRUP"]) ){
                    $row["NRORDPROMOGRUP"] = "";
                }
                if(!isset($row["NMGRUPROMOC"]) ){
                    $row["NMGRUPROMOC"] = "";
                }
                if(!isset($row["CDGRUPMUTEX"]) ){
                    $row["CDGRUPMUTEX"] = "";
                }
                if(!isset($row["IDAPLICADESC"]) ){
                    $row["IDAPLICADESC"] = "N";
                }
                if(!isset($row["IDIMPGRPROMO"]) ){
                    $row["IDIMPGRPROMO"] = "S";
                }
                if(!isset($row["DTINIVGGRPROMO"]) ){
                    $row["DTINIVGGRPROMO"] = "";
                }
                if(!isset($row["DTFINVGGRPROMO"]) ){
                    $row["DTFINVGGRPROMO"] = "";
                }

                $params[0] = array (
                    'QTPRGRUPROMIN'     => $row['QTPRGRUPROMIN'],
                    'QTPRGRUPPROMOC'    => $row['QTPRGRUPPROMOC'],
                    'NRORDPROMOGRUP'    => $row['NRORDPROMOGRUP'],
                    'CDGRUPROMOC'       => $CDGRUPO,
                    'NMGRUPROMOC'       => $row['NMGRUPROMOC'],
                    'CDGRUPMUTEX'       => $row['CDGRUPMUTEX'],
                    'IDAPLICADESC'      => $row['IDAPLICADESC'],
                    'IDIMPGRPROMO'      => $row['IDIMPGRPROMO'],
                    'DTINIVGGRPROMO'    => $row['DTINIVGGRPROMO'],
                    'DTFINVGGRPROMO'    => $row['DTFINVGGRPROMO'],
                    'CDPRODPROMOCAO'    => $row['CDPRODPROMOCAO'],
                    '__is_new'          => false,
                    'NRSEQGRPROMO'      => $NRSEQGRPROMO
                );
                
                $newRequest = new DTO\Request\DataSet(
    	            new DataSet('grupromocprod_grupopromoc', $params),
    	            $request->getMethod(),
    	            $request->getRoutePath(),
    	            $request->getUserId()
                );
                
                $newResponse = new DTO\Response();
                
                parent::save($newRequest, $newResponse);
            }
        }
        catch(\Exception $e) {
            var_dump($e->getMessage());
        }
    }

    public function AltAutoFil(DTO\Request $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $row = $rows[count($rows) - 1];

            $this->dataSourceName = "grupromocprfil";

            foreach($row["CDGRUPROMOC"] as $key => $CDGRUPO) {
                $NRSEQGRPROMOFIL = $row['NRSEQGRPROMOFIL'][$key];
                
                if(!isset($row["NRORDPROMOGRUPFIL"])) {
                    $row["NRORDPROMOGRUPFIL"] = "";
                }
                
                if(!isset($row["NMGRUPROMOC"])){
                    $row["NMGRUPROMOC"] = "";
                }
                
                if(!isset($row["CDGRUPMUTEXFIL"])){
                    $row["CDGRUPMUTEXFIL"] = "";
                }
                
                if(!isset($row["IDAPLICADESCFIL"])){
                    $row["IDAPLICADESCFIL"] = "N";
                }
                
                if(!isset($row["IDIMPGRPROMOFIL"])){
                    $row["IDIMPGRPROMOFIL"] = "S";
                }
                
                if(!isset($row["DTINIVGGRPROFIL"])){
                    $row["DTINIVGGRPROFIL"] = "";
                }
                
                if(!isset($row["DTFINVGGRPROFIL"])){
                    $row["DTFINVGGRPROFIL"] = "";
                }

                $params[0] = array(
                    'CDFILIAL'          => $row['CDFILIAL'],
                    'QTPRGRUPROMINFIL'  => $row['QTPRGRUPROMINFIL'],
                    'QTPRGRUPPROMFIL'   => $row['QTPRGRUPPROMFIL'],
                    'NRORDPROMOGRUPFIL' => $row['NRORDPROMOGRUPFIL'],
                    'CDGRUPROMOC'       => $CDGRUPO,
                    'NMGRUPROMOC'       => $row['NMGRUPROMOC'],
                    'CDGRUPMUTEXFIL'    => $row['CDGRUPMUTEXFIL'],
                    'IDAPLICADESCFIL'   => $row['IDAPLICADESCFIL'],
                    'IDIMPGRPROMOFIL'   => $row['IDIMPGRPROMOFIL'],
                    'DTINIVGGRPROFIL'   => $row['DTINIVGGRPROFIL'],
                    'DTFINVGGRPROFIL'   => $row['DTFINVGGRPROFIL'],
                    'CDPRODPROMOCAO'    => $row['CDPRODPROMOCAO'],
                    "NRORG"             => $row['NRORG'],
                    '__is_new'          => false,
                    "NRSEQGRPROMOFIL"   => $NRSEQGRPROMOFIL
                );
                
                $newRequest = new DTO\Request\DataSet(
    	            new DataSet('grupromocprfil_grupopromoc', $params),
    	            $request->getMethod(),
    	            $request->getRoutePath(),
    	            $request->getUserId()
                );
                
                $newResponse = new DTO\Response();

                parent::save($newRequest, $newResponse);
            }
        } catch(\Exception $e) {
            var_dump($e->getMessage());
        }
    }
}