<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\Controller\CrudWithParameterBag;

class SelectConsumidorMultipleFilial extends CrudWithParameterBag {
    const dataSourceConsumidor  = 'select_consumidor_by_multiple_filial';
          
    protected $parameterBagColumnsConsumidor = array('TCDFILIAL', 'CDFILIAL', 'NRORG' );
              
    public function __construct ($parameterBag, Manager $dataSourceManager, $utilService) {
        $this->parameterBag             = $parameterBag;
        $this->dataSourceManager        = $dataSourceManager;
        $this->utilService              = $utilService;
    }
    
    public function find (DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName      = self::dataSourceConsumidor;
        $this->parameterBagColumns = $this->parameterBagColumnsConsumidor;
        $conditions                = $this->utilService->getParamsAsArray($request);
        $dataSourceFilter          = SelectConsumidorMultipleFilial::prepareConsParameters($conditions);
        $requestDataSourceFilter = $request->getFilterCriteria()->getConditions();
        
        foreach($requestDataSourceFilter as $filter) {
            if($filter['operator'] === 'MAPPED_LIKE_ALL') {
                array_push($dataSourceFilter, $filter);
            }
        }
        
        $newRequest = new DTO\Request\Filter(
            new FilterCriteria(
                $this->dataSourceName,
                $dataSourceFilter,
                $request->getFilterCriteria()->getPage()
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
        
        parent::find($newRequest, $response);
    }
    
    private function prepareConsParameters($conditions) {
        if(isset($conditions['CDFILIAL'])){
            $CDFILIAL = implode('_',$conditions['CDFILIAL']);
            $TCDFILIAL = NULL;
            
        } else {
            $CDFILIAL = NULL;
            $TCDFILIAL = 'T';
        }
        
        $NRORG = $conditions['NRORG'];
        
        $parameters = array(
            'CDFILIAL'     => $CDFILIAL,
            'TCDFILIAL'    => $TCDFILIAL,
            'NRORG'        => $NRORG
        );
        
        $dataSourceFilter = array();
        
        $dataSourceFilter[0] = array(
            "columnName" => 'CDFILIAL',
            "operator"   => "=",
            "value"      => $parameters['CDFILIAL']
        ); 
        $dataSourceFilter[1] = array(
            "columnName" => 'TCDFILIAL',
            "operator"   => "=",
            "value"      => $parameters['TCDFILIAL']
        );
        $dataSourceFilter[2] = array(
            "columnName" => 'NRORG',
            "operator"   => "=",
            "value"      => $parameters['NRORG']
        );
        
        
        
        return $dataSourceFilter;
    }
}